/* jcifs smb client library in Java
 * Copyright (C) 2000  "Michael B. Allen" <jcifs at samba dot org>
 *                     "Christopher R. Hertel" <jcifs at samba dot org>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package jcifs.util;

import java.io.Writer;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Properties;

/**
 * This log utility uses a combination of bit mask control and io stream
 * functionality. All methods check the mask with the mask passed as a
 * parameter to see if any further worki (logging) should be performed. This
 * provides explicit control at runtime over what is logged. The IO stream
 * model allows the {@link java.io.PrintWriter} to be set and provides
 * many of the common {@link java.io.PrintWriter} methods that are usefull
 * for streams.
 *
 * <p>Three example log entries follow:
 *
 * <blockquote><pre>
 * Jul 7 19:17:40.999 - smb tree connect warning 
 *  path=\\140.240.194.37\IPC$
 * Jul 7 19:17:41.069 - datagram packet sent to: cranes.campus.foo.br/140.240.194.37
 * 00000: 00 06 01 10 00 01 00 00 00 00 00 00 20 43 4B 41  |............ CKA|
 * 00010: 41 41 41 41 41 41 41 41 41 41 41 41 41 41 41 41  |AAAAAAAAAAAAAAAA|
 * 00020: 41 41 41 41 41 41 41 41 41 41 41 41 41 00 00 21  |AAAAAAAAAAAAA..!|
 * 00030: 00 01                                            |..              |
 * Jul 7 19:17:47.087 - SmbFile exception
 * java.net.UnknownHostException: no name with type 0x20 with no scope for host 140.240.194.37
 *     at jcifs.netbios.NbtAddress.getAllByAddress(NbtAddress.java:618)
 *     at jcifs.smb.SmbFile.list(SmbFile.java:700)
 *     at List.main(List.java:30)
 * </pre></blockquote>
 * 
 * To use this logging facility simply add any
 * <code>Log.println</code>, <code>Log.printHexDump</code>,
 * <code>Log.printStackTrace</code>, or derived methods throughout
 * your code and set a suitable mask with <code>Log.setMask</code>. For
 * example the following mask might be set at the very beginning of your
 * program:
 * 
 * <p><blockquote><pre>
 * Log.setMask(( Log.EXCEPTIONS | foo.Log.ALL ) & ~foo.Log.LOGINS );
 * </pre></blockquote>
 *
 * This would write the stack traces from <code>Log.printStackTrace</code>
 * calls and all logging for package <code>foo</code> with the exception
 * of logging associated with the <code>LOGINS</code> mask. The following
 * is an example of using the <code>Log.println</code> method:
 *
 * <p><blockquote><pre>
 * Log.println( Log.WARNINGS, "smb tree connect warning", " path=" + path );
 * </pre></blockquote>
 *
 * <p>Masks can be combined and subtracted with one another using
 * bitwise operators. To combine two masks use the bitwise OR operator
 * <code>|</code>. The bitwise AND operator <code>&amp;</code> together with
 * bitwise NOT <code>~</code> can be used to turn off masks.
 *
 * <p>If bit manipulation is not familiar, use the <code>addMask</code>
 * and <code>subtractMask</code> methods. The equivalent of the first code
 * sample would be:
 *
 * <p><blockquote><pre>
 * Log.addMask( Log.EXCEPTIONS );
 * Log.addMask( foo.Log.ALL );
 * Log.subtractMask( foo.Log.LOGINS );
 * </pre></blockquote>
 *
 * <p>Note: When defining a new mask, one must take care not to use bits
 * that are already being used by another package. In the future, masks
 * may be statically assigned during class initialization.
 *
 * @see       jcifs.util.LogWriter
 * @see       jcifs.netbios.Log
 * @see       jcifs.smb.Log
 */

public class Log {

    // supress javadoc constructor summary by removing 'protected'
    protected Log() {}

    private static final char[] hexChars = { 
        '0', '1', '2', '3', '4', '5',
        '6', '7', '8', '9', 'A', 'B',
        'C', 'D', 'E', 'F'
    };
    private static final char[] spaceChars = {
        ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
        ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
        ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '
    };

/**
 * The systems line separator.
 */

    public static final String NL = System.getProperty( "line.separator" );

/**
 * The systems line separator length.
 */

    public static final int NL_LENGTH = NL.length();

/**
 * Mask to indicate that no messages should be logged(not even exceptions).
 */

    public static final int NONE        = 0x00000000;

/**
 * Log all messages.
 */

    public static final int ALL            = 0xFFFFFFFF;

/** 
 * This is the default mask. However, be aware that specifying a
 * mask without adding this mask will result in exceptions not being
 * logged. One should always add this EXCEPTIONS mask to the {@link
 * jcifs.util.Log#setMask(int mask)} expression unless you are explicitly
 * trying to suppress them. Of course only exceptions that go through
 * the {@link jcifs.util.Log#printStackTrace(String desc, Throwable t )}
 * facility can be filtered out.
 */

    public static final int EXCEPTIONS    = 0x10000001;

/**
 * <b>Depricated</b> <i>The jCIFS package no longer uses the
 * <code>CRITICAL_EXCEPTIONS</code> mask. It was invented to compensate
 * for a deficiency in jCIFS that has since been resolved. It will be
 * removed in the next release.</i>
 * <p>
 * Log only critical exceptions. All exceptions logged with
 * <code>printStackTrace</code> are considered critical. Exceptions are not
 * critical if they are logged with the <code>NON_CRITICAL_EXCEPTIONS</code>
 * mask.
 */

	public static final int CRITICAL_EXCEPTIONS = 0x00000001;

/**
 * <b>Depricated</b> <i>The jCIFS package no longer uses the
 * <code>NON_CRITICAL_EXCEPTIONS</code> mask. It was invented to compensate
 * for a deficiency in jCIFS that has since been resolved. It will be
 * removed in the next release.</i>
 * <p>
 * Log non-critical exceptions. The <code>EXCEPTIONS</code> mask consists of both critical and non-critical exceptions.
 */

	public static final int NON_CRITICAL_EXCEPTIONS = 0x10000000;

/**
 * Log warning messages.
 */

    public static final int WARNINGS    = 0x00000002;

/** 
 * Use this mask while actively debugging your code. It is
 * an alternative for the popular <code>System.out.println( "MADE
 * IT!" )</code> statement but can be supressed.
 */

    public static final int DEBUGGING    = 0x00000004;

/** 
 * This controls wheather or not {@link jcifs.util.Log#printHexDump}
 * messages are logged.
 */ 

    public static final int HEX_DUMPS    = 0x00000008;

/** 
 * The shared output stream of all <code>Log</code> classes. The
 * defualt is <code>System.err</code>.
 * <p>
 * <i>This was changed from System.out in jcifs-0.7.0b</i>
 */

    protected static LogWriter out = new LogWriter( System.err );

/** 
 * This is the integer mask that controls what is logged. It is shared
 * by all Log classes. The default mask is <code>EXCEPTIONS</code>
 * which consists of <code>CRITICAL_EXCEPTIONS</code> and
 * <code>NON_CRITICAL_EXCEPTIONS</code>.
 */ 

    protected static int mask = EXCEPTIONS;

    // Setting Methods

/** 
 * Specify an {@link java.io.OutputStream} to be used as the underlying
 * stream. The default <code>OutputStream</code> is
 * <code>System.out</code>. Specify a different stream such as a
 * <code>FileOutputStream</code> perhaps to divert <i>all</i> logging
 * output to a different stream.
 * 
 * @param out the stream to which logging messages are written
 */

    public static void setPrintWriter( OutputStream out ) {
        Log.out = new LogWriter( out );
    }

/** 
 * This controls what is passed to {@link java.text.SimpleDateFormat} to
 * control what the timestamp looks like in the log stream. For example, a
 * format of <code>"EEE, MMM d, h:mm:ss a"</code> would generate timestamps
 * that read <code>Tue, Mar 14, 4:57:02 PM</code>. This is a whistle.
 *
 * @param the format string
 * @see   java.util.SimpleDateFormat
 */ 

    public static void setDateFormat( String format ) {
        out.setDateFormat( format );
    }

/** 
 * Set the mask used to screen all calls to logging methods. The idea
 * here is that the mask can be specified as an arithmetic expression
 * to provide a crude but effective syslog style way to dictate what is
 * logged. For example:
 * <p><blockquote><pre>
 *  setMask( Log.EXCEPTIONS +
 *           Log.HEX_DUMPS +
 *           jcifs.netbios.Log.PACKET_DATA );
 *
 *  setMask( Log.ALL - jcifs.netbios.Log.PACKET_DIAGRAMS );
 * </pre></blockquote>
 */ 

    public static void setMask( int mask ) {
        Log.mask = mask;
    }
	public static boolean isSet( int mask ) {
		return ( Log.mask & mask ) == mask;
	}
	public static void addMask( int mask ) {
		Log.mask |= mask;
	}
	public static void subtractMask( int mask ) {
		Log.mask &= ~mask;
	}

/** 
 * Provides standard way to log any {@link java.lang.Throwable} object like
 * an {@link java.lang.Exception}.
 */
 
	public static void printStackTrace( String desc, Throwable t ) {
        if(( CRITICAL_EXCEPTIONS & mask ) != 0 )
            out.printStackTrace( desc, t );
    }
	public static void printStackTrace( int type, String desc, Throwable t ) {
        if(( type & mask ) != 0 )
            out.printStackTrace( desc, t );
    }

    // Output Methods Used Only By This Package
/** 
 * Hex dumps are ubiquitous enough to provide a standard and easy-to-use
 * method for logging them. All that is required to use this is setting
 * the mask for {@link jcifs.util.Log#HEX_DUMPS}. The hex dump output is
 * represended in the standard form and is quite efficient. Many mega bytes
 * of source data could be dumped to a file without worry.
 * 
 * <p><blockquote><pre>
 * 00000: 04 d2 29 00 00 01 00 00 00 00 00 01 20 45 47 46  |..)......... EGF|
 * 00010: 43 45 46 45 45 43 41 43 41 43 41 43 41 43 41 43  |CEFEECACACACACAC|
 * 00020: 41 43 41 43 41 43 41 43 41 43 41 41 44 00 00 20  |ACACACACACAAD.. |
 * 00030: 00 01 c0 0c 00 20 00 01 00 00 00 00 00 06 20 00  |..... ........ .|
 * 00040: ac 22 22 e1                                      |."".            |
 * </blockquote></pre>
 */

	public static void printHexDump( String desc, byte[] src ) {
		printHexDump( desc, src, 0, src.length );
	}
    public static void printHexDump( PrintStream ps, byte[] src, int srcIndex, int length ) {
        int s = length % 16;
        int r = ( s == 0 ) ? length / 16 : length / 16 + 1;
        char[] c = new char[r * (74 + NL_LENGTH)];
        char[] d = new char[16];
        int i;
        int si = 0;
        int ci = 0;
        do {
            toHexChars( si, c, ci, 5 );
            ci += 5;
            c[ci++] = ':';
            do {
                if( si == length ) {
                    int n = 16 - s;
                    System.arraycopy( spaceChars, 0, c, ci, n * 3 );
                    ci += n * 3;
                    System.arraycopy( spaceChars, 0, d, s, n );
                    break;
                }
                c[ci++] = ' ';
                i = src[srcIndex + si] & 0xFF;
                toHexChars( i, c, ci, 2 );
                ci += 2; 
                if( i < 0 || Character.isISOControl( (char)i )) {
                    d[si % 16] = '.';
                } else {
                    d[si % 16] = (char)i;
                }
            } while(( ++si % 16 ) != 0 );
            c[ci++] = ' ';
            c[ci++] = ' ';
            c[ci++] = '|';
            System.arraycopy( d, 0, c, ci, 16 );
            ci += 16;
            c[ci++] = '|';
            NL.getChars( 0, NL_LENGTH, c, ci );
            ci += NL_LENGTH;
        } while( si < length );
        ps.println( c );
	}
    public static void printHexDump( String desc, byte[] src, int srcIndex, int length ) {
        if(( HEX_DUMPS & mask ) == 0 || length == 0 ) {
            return;
        }
        int s = length % 16;
        int r = ( s == 0 ) ? length / 16 : length / 16 + 1;
        char[] c = new char[r * (74 + NL_LENGTH)];
        char[] d = new char[16];
        int i;
        int si = 0;
        int ci = 0;
        do {
            toHexChars( si, c, ci, 5 );
            ci += 5;
            c[ci++] = ':';
            do {
                if( si == length ) {
                    int n = 16 - s;
                    System.arraycopy( spaceChars, 0, c, ci, n * 3 );
                    ci += n * 3;
                    System.arraycopy( spaceChars, 0, d, s, n );
                    break;
                }
                c[ci++] = ' ';
                i = src[srcIndex + si] & 0xFF;
                toHexChars( i, c, ci, 2 );
                ci += 2; 
                if( i < 0 || Character.isISOControl( (char)i )) {
                    d[si % 16] = '.';
                } else {
                    d[si % 16] = (char)i;
                }
            } while(( ++si % 16 ) != 0 );
            c[ci++] = ' ';
            c[ci++] = ' ';
            c[ci++] = '|';
            System.arraycopy( d, 0, c, ci, 16 );
            ci += 16;
            c[ci++] = '|';
            NL.getChars( 0, NL_LENGTH, c, ci );
            ci += NL_LENGTH;
        } while( si < length );
        out.println( desc, c );
    }

/** 
 * This is an alternative to the <code>java.lang.Integer.toHexString</cod>
 * method. It is an efficient relative that also will pad the left side so
 * that the result is <code>size</code> digits.
 */ 

    public static String getHexString( int val, int size ) {
        char[] c = new char[size];
        toHexChars( val, c, 0, size );
        return new String( c );
    }
    public static String getHexString( long val, int size ) {
        char[] c = new char[size];
        toHexChars( val, c, 0, size );
        return new String( c );
    }

	public static String getHexString( byte[] src, int srcIndex, int size ) {
		char[] c = new char[size];
		size = ( size % 2 == 0 ) ? size / 2 : size / 2 + 1;
		for( int i = 0, j = 0; i < size; i++ ) {
			c[j++] = hexChars[(src[i] >> 4 ) & 0x0F];
			if( j == c.length ) {
				break;
			}
			c[j++] = hexChars[src[i] & 0x0F];
		}
		return new String( c );
	}

/** 
 * This is the same as {@link jcifs.util.Log#getHexString(int val, int
 * size)} but provides a more practical form when trying to avoid {@link
 * java.lang.String} concatenation and {@link java.lang.StringBuffer}.
 */ 

    public static void toHexChars( int val, char dst[], int dstIndex, int size ) {
        while( size > 0 ) {
            dst[dstIndex + size - 1] = hexChars[val & 0x000F];
            if( val != 0 ) {
                val >>>= 4;
            }
            size--;
        }
    }
    public static void toHexChars( long val, char dst[], int dstIndex, int size ) {
        while( size > 0 ) {
            dst[dstIndex + size - 1] = hexChars[(int)( val & 0x000FL )];
            if( val != 0 ) {
                val >>>= 4;
            }
            size--;
        }
    }

/** 
 * Print an int. The bits of the integer <code>type</code> must match the
 * {@link jcifs.util.Log#mask}.
 */

    public static void println( int type, String desc, int x ) {
        if(( type & mask ) != 0 )
            out.println( desc, x );
    }

/** 
 * Print a char[] array. The bits of the integer <code>type</code> must
 * match the {@link jcifs.util.Log#mask}.
 */
 
    public static void println( int type, String desc, char x[] ) {
        if(( type & mask ) != 0 )
            out.println( desc, x );
    }

/** 
 * Print a {@link java.lang.String}. The bits of the integer
 * <code>type</code> must match the {@link jcifs.util.Log#mask}.
 */
 
    public static void println( int type, String desc, String x ) {
        if(( type & mask ) != 0 )
            out.println( desc, x );
    }

/** 
 * Print an {@link java.lang.Object}. The bits of the integer
 * <code>type</code> must match the {@link jcifs.util.Log#mask}.
 */
 
    public static void println( int type, String desc, Object x ) {
        if(( type & mask ) != 0 )
            out.println( desc, x );
    }

/**
 * Print the contents of a {@link java.util.Properties} object to the
 * log stream using the <code>store</code> method.
 */

	public static void printProperties( int type, String desc, Properties p ) {
        if(( type & mask ) != 0 )
            out.printProperties( desc, p );
	}
}

